# Fuel selection algorithm

ACCESS_SAS_fuel_selection = function(input_fuel_prices, gea_scenario='m') {
  
  # Object passed to data preparation script calling given GEA scenario (high, medium low)
  gdp = gea_scenario
  
  #############################################################################
  ######## create an array with data needed to calculate demands  #############
  #############################################################################
  
  ### define years
  policy_years <- as.character(seq(2020,2100,10))
  years_2030 <- as.character(seq(2030,2090,10))
  
  ### Necessary vectors
  exp_grp <- c('R1','R2','R3','R4','U1','U2','U3','U4')
  stoves <- c('frwd_three','frwd_inbms','frwd_ifbms','kero_parst','lpgg_gasst','pgas_pgsst','elec_elest')
  adv_stv <- c('kero_parst','lpgg_gasst','pgas_pgsst','elec_elest')
  stv_eff <- c('frwd_three'=0.15,'frwd_inbms'=0.25,'frwd_ifbms'=0.35,'kero_parst'=0.45,'lpgg_gasst'=0.60,'pgas_pgsst'=0.60,'elec_elest'=0.80)
  variables <- c('exp_ppp_pc','exp_ppp_hh','disc_rate','hh_size','ck_dmd','population','hh_num','cook_ratio','cook_exp')
  stv_var <- c('lfyr','an_stv','fuel_price','cook_price')
  pp_var <- c('pp_diff','pp_ratio')
  dmd_var <- c('orig_dmd','demand_UE')
  fuels <- c('frwd','kero','lpgg','pgas','elec')
  years <- as.character(c(2005,seq(2010,2100,10)))
  coeff <- c('a','b')
  units <- c('GJ','GWyr','GWyr_FE')
  tier1 <- c('lpgg_gasst','pgas_pgsst','elec_elest')
  tier2 <- c('kero_parst')
  tier3 <- c('frwd_three','frwd_inbms','frwd_ifbms')
  
  ### dynamic groups
  dyn_grp <- c('R4'='exp_grpR500','U4'='exp_grpU500')
  
  ### stv_array[stoves,variables,exp_grp,years]
  stv_coeff_array <- array(NA, dim=c(length(exp_grp),length(tier1),length(coeff),length(years)),dimnames=list('exp_grp'=exp_grp,'stoves'=tier1,'coeff'=coeff,'years'=years))
  dem_array    <- array(NA, dim=c(length(exp_grp),length(stoves),length(dmd_var),length(years)),dimnames=list('exp_grp'=exp_grp,'stoves'=stoves,'variables'=dmd_var,'years'=years))
  MESSAGE_demand  <- array(NA, dim=c(length(exp_grp),length(stoves),length(years),length(units)),dimnames=list('exp_grp'=exp_grp,'stoves'=stoves,'years'=years,'units'=units))
  constraint<- array(NA, dim=c(length(exp_grp),length(stoves),length(years)),dimnames=list('exp_grp'=exp_grp,'stoves'=stoves,'years'=years))
  
  #########################################################################
  ###################### Create necessary data input objects #############
  #########################################################################
  
  # Following portion of code generates the 'grp_array', 'stv_array', 'constraint', and 'policy_array' objects
  
  # Create empty placeholder objects
  grp_array       <- array(NA, dim=c(length(exp_grp),length(variables),length(years)),dimnames=list('exp_grp'=exp_grp,'variables'=variables,'years'=years))
  stv_array       <- array(NA, dim=c(length(exp_grp),length(stoves),length(stv_var),length(years)),dimnames=list('exp_grp'=exp_grp,'stoves'=stoves,'variables'=stv_var,'years'=years))
  fuel_adj_array  <- array(NA, dim=c(length(exp_grp),length(fuels)),dimnames=list('exp_grp'=exp_grp,'fuels'=fuels))
  new_years <- as.character(seq(2010,2100,10))
  
  ### function to adjust data into an usable format
  adj_df <- function(df_data) 
  { rownames(df_data) <- df_data[,1]
  { if (ncol(df_data)==2) { df_data <- data.frame(df_data[,-1,drop=FALSE]) }
  else { df_data <- data.frame(df_data[,-1]) } }
  colnames(df_data) <- gsub('X','',colnames(df_data))
  df_data }
  
  #----------------------------------
  ### Load/create projections for HH characteristics, by expenditure group and year
  #----------------------------------
  
  ### expenditure per capita in ppp
  v <- 'exp_ppp_pc'
  exp_ppp_pc_RU <- adj_df(read.table(paste0('inputs/exp_ppp_pc_',gdp,'.txt'), header=TRUE, sep='\t'))
  grp_array[,v,] <- as.matrix(exp_ppp_pc_RU)
  
  ### hh size
  v <- 'hh_size'
  hh_size_2005 <- read.table('inputs/hh_size_2005.txt', header=TRUE, sep='\t')
  grp_array[,v,] <- hh_size_2005[,v]
  reg_hh_size <- read.table('inputs/regress_hh_size.txt', header=TRUE, sep='\t')
  y <- new_years
  for (d in names(dyn_grp)) {
    grp_array[d,v,y] <- reg_hh_size[match(dyn_grp[[d]],reg_hh_size[,'rururb']),'b']*log(grp_array[d,'exp_ppp_pc',y])+
      reg_hh_size[match(dyn_grp[[d]],reg_hh_size[,'rururb']),'a']
  }
  
  ### if the new household size is LESS THAN than 2, it's adjusted to 2 
  grp_array[,v,][grp_array[,v,]<2] <- 2
  
  ### hh expenditure ###
  grp_array[,'exp_ppp_hh',] <- grp_array[,'exp_ppp_pc',]*grp_array[,'hh_size',]
  
  ### discount rate ###
  v <- 'disc_rate'
  disc_rate_2005 <- read.table('inputs/disc_rate_2005.txt', header=TRUE, sep='\t')
  grp_array[,v,] <- disc_rate_2005[,v] 
  d <- names(dyn_grp)
  grp_array[d,v,new_years] <- -0.162*log(grp_array[d,'exp_ppp_hh',new_years])+1.9558
  
  ### cook ratio ###
  v <- 'cook_ratio'
  cook_ratio_2005 <- read.table('inputs/cook_ratio_2005.txt', header=TRUE, sep='\t')
  grp_array[,v,] <- cook_ratio_2005[,v]
  reg_cook_ratio <- read.table('inputs/regress_cook_ratio.txt', header=TRUE, sep='\t')
  y <- new_years
  for (d in names(dyn_grp)) {
    grp_array[d,v,y] <- reg_cook_ratio[match(dyn_grp[[d]],reg_cook_ratio[,'rururb']),'b']*log(grp_array[d,'exp_ppp_hh',y])+
      reg_cook_ratio[match(dyn_grp[[d]],reg_cook_ratio[,'rururb']),'a']
  }
  
  ### cooking expenditure ###
  v <- 'cook_exp'
  grp_array[,v,] <- grp_array[,'cook_ratio',]*grp_array[,'exp_ppp_hh',]
  for(d in names(dyn_grp)) {
    max_year <- names(which.max(grp_array[d,v,]))
    if (max_year!='2100')
    { adj_year <- as.character(seq(as.numeric(max_year),2100,10))
    grp_array[d,'cook_ratio',adj_year] <- grp_array[d,'cook_ratio',max_year]
    grp_array[d,v,adj_year] <- grp_array[d,'cook_ratio',adj_year]*grp_array[d,'exp_ppp_hh',adj_year] } 
  }
  
  ### cooking demand ###
  v <- 'ck_dmd'
  dmd_2005 <- read.table('inputs/ck_dmd_2005.txt', header=TRUE, sep='\t')
  grp_array[,v,] <- dmd_2005[,v]
  reg_dmd <- read.table('inputs/regress_demand.txt', header=TRUE, sep='\t')
  y <- new_years
  for (d in names(dyn_grp)) {
    grp_array[d,v,y] <- (reg_dmd[match(dyn_grp[[d]],reg_dmd[,'rururb']),'b']*log(grp_array[d,'exp_ppp_pc',y])+
                           reg_dmd[match(dyn_grp[[d]],reg_dmd[,'rururb']),'a'])*grp_array[d,'hh_size',y]
  }
  
  ### population data from text file ###
  population <- adj_df(read.table(paste0('inputs/population_',gdp,'.txt'), header=TRUE, sep='\t'))
  grp_array[,'population',] <- as.matrix(population)
  grp_array[,'hh_num',] <- grp_array[,'population',]/grp_array[,'hh_size',]
  
  #----------------------------------
  ### Stove costs array
  #----------------------------------
  
  ### stove characteristics ###
  stove_char <- adj_df(read.table('settings/stv_char.txt', header=TRUE, sep='\t'))
  stove_char$fuel <- as.character(stove_char$fuel)
  ### life year of stoves ###
  lfyr_2005 <- stove_char$lfyr
  for (g in exp_grp) { stv_array[g,,'lfyr',] <- as.matrix(lfyr_2005) }
  
  ### annualised stove price ###
  v<-'an_stv'
  for (s in stoves) {
    stv_array[,s,v,] <- (stove_char[s,'price']*grp_array[,'disc_rate',])/(1-(1+grp_array[,'disc_rate',])^(-stv_array[,s,'lfyr',]))
  }
  
  ### fuel price ###
  v <- 'fuel_price'
  prices_2005 <- adj_df(read.table('settings/fuel_price_2005.txt', header=TRUE, sep='\t'))
  fuel_adj_2005 <- read.table('settings/fuel_adj_2005.txt', header=TRUE, sep='\t')
  options(warn=-1)
  fuel_adj_2005[,'factor'] <- as.numeric(as.character(fuel_adj_2005[,'factor']))
  options(warn= 1)
  for (f in fuels) {
    for (g in exp_grp) {
      fuel_adj_array[g,f] <- fuel_adj_2005[fuel_adj_2005[,'fuel']==f & fuel_adj_2005[,'grp']==g,][['factor']]
    }
  }
  for (s in stoves) {
    stv_array[,s,v,'2005'] <- prices_2005[stove_char[s,'fuel'],]*fuel_adj_array[,stove_char[s,'fuel']]
    for (y in new_years) {
      stv_array[,s,v,y] <- input_fuel_prices[stove_char[s,'fuel'],y]*fuel_adj_array[,stove_char[s,'fuel']]
    }
  }
  
  #----------------------------------
  ### Create policies array
  #----------------------------------
  
  # create copy of stv_array
  stv_array_old <- stv_array
  
  # Access Policy Array
  ss_names <- c('nss','fss','lss','pss','ess')
  fs_names<-c('lfs','pfs','efs')
  policy_stoves <- c('nss'='frwd_inbms','fss'='frwd_ifbms','lss'='lpgg_gasst','pss'='pgas_pgsst', 'ess'='elec_elest','lfs'='lpgg_gasst','pfs'='pgas_pgsst','efs'='elec_elest')
  
  # Load policy settings and create 'policy_array' object
  temp = read.table('settings/policy_assumptions.txt', header=T, sep='\t')
  policy_array = as.matrix(temp[,-1])
  dimnames(policy_array) = list(policy=temp$policy, exp_grp=names(temp)[-1])
  
  # Stove Subsidies
  for (p in ss_names){
    s<-policy_stoves[p]
    for (g in exp_grp){
      for (y in policy_years){
        stv_array[g,s,'an_stv',y]<-(stove_char[s,'price']*((100-policy_array[p,g])/100)*grp_array[g,'disc_rate',y])/
          (1-(1+grp_array[g,'disc_rate',y])^(-stv_array[g,s,'lfyr',y]))
      }
    }
  }
  
  # Fuel Price Subsidies
  for (p in fs_names){
    s<-policy_stoves[p]
    for (g in exp_grp){
      if(policy_array[p,g]>0){
        for (y in policy_years){
          stv_array[g,s,'fuel_price',y]<-stv_array_old[g,s,'fuel_price',y]*(1-policy_array[p,g]/100)
        }
      }
    }
  }
  
  #----------------------------------
  
  ### total cooking price ###
  v <- 'cook_price'
  for (s in stoves) {
    stv_array[,s,v,] <- (stv_array[,s,'fuel_price',]/stove_char[s,][['eff']]) +
      (stv_array[,s,'an_stv',]/grp_array[,'ck_dmd',])
  }
  
  #----------------------------------
  
  ### Fill constraint array
  for (s in stoves){
    constraint_temp <- read.table(paste('constraints/',s,'_constraint.txt',sep=''), header=T, sep='\t',row.names=1)
    colnames(constraint_temp)<- c(2005,2010,2020,2030,2040,2050,2060,2070,2080,2090,2100)
    for (g in exp_grp){
      for (y in years){
        constraint[g,s,y] <- constraint_temp[g,y]
      }
    }
  }
  
  ######################################################################################
  ######## START of actual fuel selection algorithm to compute demands by groups #######
  ######################################################################################
  
  ### create a matrix with demand curve coefficients ###
  coeff_exp_grp <- read.table('inputs/coeff_exp_grp_2005.txt', header=TRUE, sep='\t')
  for (s in tier1) {
     for (g in exp_grp) {
        stv_coeff_array[g,s,'a',] <- coeff_exp_grp[coeff_exp_grp[,'stove']==s & coeff_exp_grp[,'grp']==g,][['a']]
        stv_coeff_array[g,s,'b',] <- coeff_exp_grp[coeff_exp_grp[,'stove']==s & coeff_exp_grp[,'grp']==g,][['b']]
     }
  }
  
  ### calculate unadjusted demands using demand curves ###
  v <- 'orig_dmd'
  for (s in tier1) {
  		dem_array[,s,v,] <- (stv_array[,s,'cook_price',]/stv_coeff_array[,s,'a',])^(1/stv_coeff_array[,s,'b',])
  		## if demands are NA, its replaced by 0
  		dem_array[,s,v,][is.na(dem_array[,s,v,y])] <- 0 
  }
  
  #### ASSUME ALL NEW COOKING EXPENDITURE IN DYNAMIC GROUPS IS ALLOCATED TO TIER 1 FUELS ####
  for (s in tier1) {
  	dem_array[,s,v,] <- dem_array[,s,v,]+((grp_array[,'cook_exp',]-grp_array[,'cook_exp','2005'])/stv_array[,s,'cook_price',])
  }
  
  ###########################################################################################
  ################################ Tier System ##############################################
  ###########################################################################################
  
  ############################## Tier 1 Demand ##############################
  d<-'demand_UE'
  o<-'orig_dmd'
  
  ## Tier 1 cooking price array
  tier1_price <- array(NA, dim=c(length(exp_grp),length(tier1),length(years)),
                           dimnames=list('exp_grp'=exp_grp,'stoves'=tier1,'years'=years))
  for (s in tier1){
  	tier1_price[,s,] <- stv_array[,s,'cook_price',]
  }
  
  ### Calculate Tier 1 Demand
  for (g in exp_grp){
  	for (y in years){
  		totdmd<-0
  		# set s = cheapest fuel for that tier, exp_grp, and year
  		s <- names(tier1_price[g,,y])[order(tier1_price[g,,y])[1]]
  		
  		########################################### 5% BUFFER ##########################
  		#tier1_price[g,s,y]>0 && 
  		if (y!='2005'){
  				if ((abs(tier1_price[g,s,y]-tier1_price[g,s_old,y])/tier1_price[g,s,y])<0.05){s<-s_old}
  		}
  		
  		##################################################################################
  		dem_array[g,s,d,y]<-min(dem_array[g,s,o,y],constraint[g,s,y],grp_array[g,'ck_dmd',y])
  		totdmd<-dem_array[g,s,d,y]
  		s_old <- s
  		
  		# loop through stove types in that tier from 2nd cheapest to most expensive
  		tcp <- tier1_price[g,,y][names(tier1_price[g,,y])!=s]
  		for (i in order(tcp)) {
  			s2<-names(tcp)[i]			
  			if (min(constraint[g,s2,y],dem_array[g,s2,o,y])<=totdmd){
  				dem_array[g,s2,d,y]<-0
  			} else if (dem_array[g,s2,o,y]<min(constraint[g,s2,y],grp_array[g,'ck_dmd',y])){
  				dem_array[g,s2,d,y]<-dem_array[g,s2,o,y]-totdmd
  			} else {dem_array[g,s2,d,y]<-min(constraint[g,s2,y],grp_array[g,'ck_dmd',y])-totdmd}
  			totdmd<-totdmd+dem_array[g,s2,d,y]
  		}
  	}
  }
  
  # Total Tier 1 demand
  tier1_totdmd<-array(NA, dim=c(length(exp_grp),length(years)),
                           dimnames=list('exp_grp'=exp_grp,'years'=years))
  for (y in years){
  	for (g in exp_grp){
  		tier1_totdmd[g,y]<-sum(dem_array[g,,d,y],na.rm=T)
  	}
  }
  
  # Original Demand Array
  orig_t1_dem<-dem_array[,,d,]
  
  # Make adjustment for kerosene share...
  kero_share_2005 = read.table('inputs/kero_share_2005.txt', skip=1) # Pre-computed from 2005 India survey
  kero_share = kero_share_2005[,2]
  names(kero_share) = kero_share_2005[,1]
  k<-'kero_parst'
  for (g in exp_grp){
  	for (y in years){
  		dem_array[g,k,d,y]<-(grp_array[g,'ck_dmd',y]-tier1_totdmd[g,y])*kero_share[g]
  	}
  }
  
  # Original kerosene demand
  orig_k_dem<-dem_array[,k,d,]
  
  # Total advanced stove demand
  tier12_totdmd<-array(NA, dim=c(length(exp_grp),length(years)),
                           dimnames=list('exp_grp'=exp_grp,'years'=years))
  for (y in years){
  	for (g in exp_grp){
  		tier12_totdmd[g,y]<-sum(dem_array[g,,"demand_UE",y],na.rm=T)
  	}
  }
  
  ### Tier 3 cooking price array
  tier3_price <- array(NA, dim=c(length(exp_grp),length(tier3),length(years)),
                           dimnames=list('exp_grp'=exp_grp,'stoves'=tier3,'years'=years))
  for (s in tier3){
  	tier3_price[,s,] <- stv_array[,s,'cook_price',]
  }
  
  ############################## Tier 3 Demand ##############################
  ### Calculate Tier 3 demand
  for (g in exp_grp){
  	for (y in years){
  		totdmd<-0
  		# set s = cheapest fuel for that tier, exp_grp, and year
  		s <- names(tier3_price[g,,y])[order(tier3_price[g,,y])[1]]
  		dem_array[g,s,d,y]<-min(constraint[g,s,y],(round(grp_array[g,'ck_dmd',y],4)-round(tier12_totdmd[g,y],4)))
  		totdmd<-dem_array[g,s,d,y]
  		for (i in order(tier3_price[g,,y])[2:length(tier3)]){
  				s <-  names(tier3_price[g,,y])[i]
  				dem_array[g,s,d,y]<-min(constraint[g,s,y],(round(grp_array[g,'ck_dmd',y],4)-round(tier12_totdmd[g,y],4)-totdmd))
  				totdmd<-totdmd+dem_array[g,s,d,y]
  		}
  	}
  }
  
  # Original tier 3 demands
  orig_t3_dem <- dem_array[,,d,]
  
  ######################################## Price Upset Adjustments #####################################
  # If kerosene is more expensive than the cheapest tier 1 fuel, kerosene demand is met with tier 1 fuel
  # Treat 2020 differently to avoid complications with constraints - assume poor users switching from kerosene don't have access to infrastructure
  
  y<-'2020'
  l<-'lpgg_gasst'
  k<-'kero_parst'
  for (g in exp_grp){
  	if (stv_array[g,k,'cook_price',y]>stv_array[g,l,'cook_price',y]){
  		dem_array[g,k,d,y]<-0
  		dem_array[g,l,d,y]<-orig_t1_dem[g,l,y]+orig_k_dem[g,y]
  	}
  }
  
  # 2030 on, no constraints means any tier 1 fuel can substitute for kerosene
  for (g in exp_grp){
  	for (y in years_2030){
  		if (stv_array[g,k,'cook_price',y]>min(tier1_price[g,,y])){
  			s <- names(tier1_price[g,,y])[order(tier1_price[g,,y])[1]]
  			if(constraint[g,s,y]<grp_array[g,'ck_dmd',y]){s<-names(tier1_price[g,,y])[order(tier1_price[g,,y])[2]]}
  			if(constraint[g,s,y]<grp_array[g,'ck_dmd',y]){s<-names(tier1_price[g,,y])[order(tier1_price[g,,y])[3]]}
  			dem_array[g,k,d,y]<-0
  			dem_array[g,s,d,y]<-min(orig_t1_dem[g,s,y]+orig_k_dem[g,y],grp_array[g,'ck_dmd',y],constraint[g,s,y])
  		}
  	}
  }
  t1_dem2<-dem_array[,,d,]
  
  # If biomass is more expensive than tier 1, urban groups switch to tier 1, rural groups collect biomass
  u_grp<-c('U1','U2','U3','U4')
  y<-'2020'
  l<-'lpgg_gasst'
  for (g in u_grp){
  	if (min(tier3_price[g,,y]>tier1_price[g,l,y])){
  		t3<-names(tier3_price[g,,y])[order(tier3_price[g,,y])[1]]
  		dem_array[g,t3,d,y]<-0
  		dem_array[g,l,d,y]<-t1_dem2[g,l,y]+orig_t3_dem[g,t3,y]
  	}
  }
  for (g in u_grp){
  	for (y in years_2030){
  		if (min(tier3_price[g,,y]>min(tier1_price[g,,y]))){
  			t1<-names(tier1_price[g,,y])[order(tier1_price[g,,y])[1]]
  			t3<-names(tier3_price[g,,y])[order(tier3_price[g,,y])[1]]
  			dem_array[g,t3,d,y]<-0
  			dem_array[g,t1,d,y]<-t1_dem2[g,t1,y]+orig_t3_dem[g,t3,y]
  		}
  	}
  }
  
  ###########################################################################################
  ###########################################################################################
  
  ### adjust if estimate exceeds expected demand
  MESSAGE_demand[,,,'GJ'] <- dem_array[,,'demand_UE',]
  for (y in years) {
     ## set all the negative values to zero
     MESSAGE_demand[,,y,'GJ'][MESSAGE_demand[,,y,'GJ']<0] <- 0
  }
  
  ### convert to GWyr and multiply by number of hhs ###
  for (s in stoves) { MESSAGE_demand[,s,,'GWyr']<- MESSAGE_demand[,s,,'GJ']*grp_array[,'hh_num',]*1e6 }
  MESSAGE_demand[,,,'GWyr'] <- MESSAGE_demand[,,,'GWyr']*(1/3.6)*(1/1000)*(1/8760)
  for (grp in exp_grp) { MESSAGE_demand[grp,,,'GWyr_FE'] <- MESSAGE_demand[grp,,,'GWyr']/stv_eff }
  
  ###############################################################################################
  ### RETURN OUTPUT DEMANDS
  ###############################################################################################
  
  # This is the effective demand output by expenditure, group, and period: 'MESSAGE_demand'
  # It contains information in 'dem_array' plus cleanup, conversion, and addition of final energy GJ
  # This data should be fed back into MESSAGE/IAM iteratively until some convergence criterion achieved
  return(MESSAGE_demand)

}

#-----------------
# END
#-----------------